/* File: cdm_DAQ.cpp
   Angel Perles. 2010/5/30
   DAQ functions
*/

#include <cdm_Error.h>
#include <cdm_Link.h>

#include <stdio.h>
#include <string.h>

#define MSG_BUF_SIZE    100 // size of the temporal buufer for message creation and reception


/*****************************************************************************/
cdm_Error cdm_DAQWriteDigitalLine(int subdevice, int line, int value) {

    char buff[MSG_BUF_SIZE];
    cdm_Error cdm_result;

    // (here, posible asynchronous treatment of signals)

    // test parameters
    if ((subdevice < 0) || (subdevice > 255)) {
        return(CDM_Error_DAQBadSubdevice);
    }
    if ((line < 0) || (line > 32)) {
        return(CDM_Error_DAQBadLine);
    }
    if ((value !=0) && (value != 1)) {
        return(CDM_Error_DAQBadValue);
    }

    // clear reception buuffer
    cdm_LinkRXClearBuffer();

    // compose message, i.e "#DOL02021\x0D";
    sprintf(buff,"#DOL%02x%02x%d\x0D",subdevice,line,value);

    // send mesage
    cdm_result = cdm_LinkTXStr(buff);
    if (cdm_result != CDM_Error_NoError) {
        return(cdm_result);
    }

    // wait response, THIS COULD BE ASYNCHRONOUS IN THE FUTURE
    cdm_result = cdm_LinkRXMessage(buff);
    if (cdm_result != CDM_Error_NoError) {
        return(cdm_result);
    }

    // test the response, ANOTHER DAY
    // ...

    return(CDM_Error_NoError);
}

/*****************************************************************************/
cdm_Error cdm_DAQReadDigitalFrecuency(int subdevice, double *value) {

    char buff[MSG_BUF_SIZE];
    cdm_Error cdm_result;

    // (here, posible asynchronous treatment of signals)

    // test parameters
    if ((subdevice < 0) || (subdevice > 255)) {
        return(CDM_Error_DAQBadSubdevice);
    }

    // clear reception buuffer
    cdm_LinkRXClearBuffer();

    // compose message, i.e "#DIF10\x0D"
    sprintf(buff,"#DIF%02d\x0D",subdevice);

    // send mesage
    cdm_result = cdm_LinkTXStr(buff);
    if (cdm_result != CDM_Error_NoError) {
        return(cdm_result);
    }

    // wait response, THIS COULD BE ASYNCHRONOUS IN THE FUTURE
    cdm_result = cdm_LinkRXMessage(buff);
    if (cdm_result != CDM_Error_NoError) {
        return(cdm_result);
    }

    // test the response
    if (strlen(buff) == 11) {
        int i;
        sscanf(buff+1,"%d",&i);
        *value = i;
    } else {
        return(CDC_Error_UnimplementedError);
    }

    return(CDM_Error_NoError);

   //SendMsg("#DIF10\x0D");
}

/*****************************************************************************/
cdm_Error cdm_DAQWritePWM(int subdevice, double dutty_cycle) {

    char buff[MSG_BUF_SIZE];
    cdm_Error cdm_result;
    int dummy;

    // (here, posible asynchronous treatment of signals)

    // test parameters
    if ((subdevice < 0) || (subdevice > 255)) {
        return(CDM_Error_DAQBadSubdevice);
    }

    // clear reception buffer
    cdm_LinkRXClearBuffer();

    // compose message, i.e "#PWM300976\x0D"
    dummy = dutty_cycle * 10.0;
    sprintf(buff,"#PWM%02d%04d\x0D",subdevice,dummy);

//return(CDM_Error_NoError);

    // send mesage
    cdm_result = cdm_LinkTXStr(buff);
    if (cdm_result != CDM_Error_NoError) {
        return(cdm_result);
    }

    // wait response, THIS COULD BE ASYNCHRONOUS IN THE FUTURE
    cdm_result = cdm_LinkRXMessage(buff);

    return(cdm_result);
}
