/* CD_AnalogInput.c 
   Analog inputs
   Angel Perles. June 2009.

   used resources
    P2.5 is the potentiometer
    ADC0

   dependencies

   IMPORTANT:

*/

#include <C8051F340.h>

#include "CD_Main.h"
#include "CD_AnalogInput.h"
#include "CD_tools.h"
#include "CD_Link.h"


#define VOLT_REF        3300    /* reference voltage in mV */
#define ADCRESOL     1024    /* 2^nbits of resolution of the ADC */

#define ADC_CLOCK       1000000         // SAR clock

sfr16 ADC0     = 0xBD;                 // ADC result


/****************************************************************************/
void CD_AnalogInput_Init (void) {

   ADC0CN = 0x00;                      // Disable ADC0
                                       // Normal Track Mode
                                       // Conversion started writing to AD0BUSY
   ADC0CN |= 0x40;  // Change to low power tracking mode ADCOTM=1 to allow better settling timing of the multiplexer 

   //AMX0P = 0x0D;                       // P2.5 is the positive input
   AMX0N = 0x1F;                       // GND is the negative input

   // ADC conversion clock <= 3MHz
   ADC0CF = (SYSCLK/ADC_CLOCK) << 3;   

   ADC0CF &= ~0x04;                    // Make ADC0 right-justified

   REF0CN = 0x0A;                      // VREF = VDD, bias generator is on.

   AD0EN = 1;                          // Enable ADC0

}

/****************************************************************************/
void CD_AIS (unsigned char *msg) large {

    unsigned char subdevice;
   //unsigned int ADC_result;

    unsigned char line;

    unsigned long val;

    unsigned char i;

    char num[10];

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(0x00);  /* bad subdevice */
        return; 
    };

    /* test subdevice numbre */
    if (subdevice != 0x40) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    if (CD_HexaStrTo_u8(msg+2, 2, &line)) {
        CD_LinkMsgError(CD_Err_BadValue);  /* bad line */
        return; 
    };


   AMX0P = line;    // set the multiplexer line 

   AD0INT = 0;  // clear end of conversion flag
   AD0BUSY = 1; // start conversion

   while (AD0INT == 0){};   // wait end of conversion

   val = ADC0;
   val = (val * VOLT_REF) / 1024;   /* convert to mV */


    //val=1234567890;

    for (i=0;i<5;i++) {
        num[i]='0'+ val % 10;
        val /= 10;
    }

    CD_LinkTXStr("#!");
    for (i=0;i<5;i++) {
        CD_LinkTXChar(num[4-i]);
    }
    CD_LinkTXChar('\x0D');

}

