/* Cd_DigitalFrecInput.c 
   Digital frecuency measurement
   Angel Perles. June 2009.

   used resources
     timer1
     int0
     P0.3 (asigned to int0)
   dependencies
     FrecInput shares timer scaler

   IMPORTANT:
     ISR race conditions must be solved mes endevant
*/

#include <C8051F340.h>

#include "CD_Main.h"

volatile unsigned int cuenta_desbordes;
volatile unsigned int cp_cuenta_desbordes;
volatile unsigned char cp_TH1,cp_TL1;
volatile unsigned char lock_DIF;

/****************************************************************************/
void CD_DigitalFrecInput_Init (void) 
{

	// desconectar interrupciones
	EA = 0;

    CKCON &= ~0x08; // timer 1 uses preescaler as defined for timer 0, that is 48 MHz/48 -> 1uS

	// programar el sistema del timer 1
	TR1=0;
    TF1=0;

	TMOD &= 0x0F;
	TMOD |= 0x10;	// control soft, modo 1, temporizador
    TH1=0;
	TL1=0;

    cuenta_desbordes = 0;

	ET1 = 1;

	// programar la interrupcion externa 0

	IT0 = 1;    // edge trigered
	IE0 = 0;

    IT01CF &= ~0x07; 
    IT01CF |= 0x03; /* P0.3 will be interrupt 0 input */

	EX0 = 1;    // enable INT0 interrupt

	// conectar interrupciones
	EA = 1;

    lock_DIF = 0;

    TR1 = 1;    /* start TIMER 1 */
}

/****************************************************************************/
void CD_DIF (unsigned char *msg) large {

    unsigned char subdevice;
    unsigned long val;
    unsigned char i;

    char num[10];

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    /* test subdevice numbre */
    if (subdevice != 0x10) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };


    lock_DIF = 1;   /* avoid race condition */

    val = cp_cuenta_desbordes;
    val <<= 8;
    val |= cp_TH1;
    val <<= 8;
    val |= cp_TL1;

    lock_DIF = 0;

    if (val > 1000000L) {  // 1Hz
        val = 1;
    } else {
        val = 1000000L/val;
    }



    //val=1234567890;

    for (i=0;i<10;i++) {
        num[i]='0'+ val % 10;
        val /= 10;
    }

    CD_LinkTXStr("#!");

    for (i=0;i<10;i++) {
        CD_LinkTXChar(num[9-i]);
    }

    CD_LinkTXChar('\x0D');

}

/****************************************************************************/
void timer1_ISR (void) interrupt 3 {   //para solucionar los 0v tenemos que contar los debordes a ojo para para obligar un flag de bajada

    if (cuenta_desbordes != 65535) {    // limitar los desbordes
        cuenta_desbordes++;
    }
}

/****************************************************************************/
void int0_ISR (void) interrupt 0 {

    TR1 = 0;

    if (lock_DIF == 0) {
        cp_TH1 = TH1;
	    cp_TL1 = TL1;
	    cp_cuenta_desbordes = cuenta_desbordes;
    }

    TH1 = 0;
    TL1 = 0;
    cuenta_desbordes = 0;

    TR1 = 1;
}

