/* Cd_DigitalFrecOutput.c 
   Digital frecuenci signals generation 
   Angel Perles. June 2009.

   used resorces

    timer 0.
    line P2^3

*/

#include <C8051F340.h>

#include "CD_Main.h"

sbit linea_servo=P2^3;

volatile unsigned char high_h, high_l, low_h, low_l; /* timer reload values */


/****************************************************************************/
void CD_DigitalFrecOutput_Init (void) 
{

    TR0=0;
    TF0=0;

	EA=0; //Deshabilitar las interrupcion
		//Programar el Timer 0

    CKCON &= 0xF8;  // para el Silabs C8051F340, 48 Mhz /48 -> 1uS por cuenta
    CKCON |= 0x02;
	
    TR0=0;
	TF0=0;
	TMOD&=0xF0;
	TMOD|=0x01;

	//Habilitar las interrupciones
	ET0=1;
	EA=1;
}

/****************************************************************************/
void CD_DOF (unsigned char *msg) large {

    unsigned char subdevice;
    unsigned int val;

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    /* test subdevice numbre */
    if (subdevice != 0x20) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    /* high time in uS*/
    if (CD_HexaStrTo_u16(msg+2, 4, &val)) {
        CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
        return; 
    };

    val = 65536 - val;
	high_l = val & 0xFF;
	high_h = val >>8;

    /* low time in uS*/
    if (CD_HexaStrTo_u16(msg+6, 4, &val)) {
        CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
        return; 
    };

    val = 65536 - val;
	low_l = val & 0xFF;
	low_h = val >> 8;


	//Cargo la 1 Cuenta y arranco
	linea_servo = 1;
	TL0=high_l;
	TH0=high_h;
	TR0=1;
    
    CD_LinkMsgOK();    
}

/****************************************************************************/
void Timer0_ISR (void) interrupt 1
{
	if(linea_servo==1) {
        //Hacer la tarea 
	    linea_servo = 0;
        TR0=0;
        //Reprogramar el Timer
	    TL0=low_l;
	    TH0=low_h;
	    TR0=1;
	} else {
        //Hacer la tarea 
	    linea_servo = 1;
        //Reprogramar el Timer
	    TR0=0;
	    TL0=high_l;
	    TH0=high_h;
	    TR0=1;    
	}
}


/* end */
