/* Cd_DigitalIO.c 
   Digital input/output module
   Angel Perles. June 2009.

   used resources
    All ports

   dependencies


   IMPORTANT:

*/

#include "CD_Main.h"

#include <stdio.h>
#include <string.h>

#include <C8051F340.h>


sbit Led1 = P2^2;   // LED='1' ON
sbit Sw1  = P2^0;



/****************************************************************************/
/* DigitalOutputLine                                                        */
void CD_DOL(unsigned char *msg) large {
    
    unsigned char subdevice;
    unsigned char line;
    unsigned char value;

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    if (CD_HexaStrTo_u8(msg+2, 2, &line)) {
        CD_LinkMsgError(CD_Err_BadLine);  /* bad line */
        return; 
    };

    if (CD_HexaStrTo_u8(msg+4, 1, &value)) {    /* ARRGELAR PER PERFORMANCE MES ENDEVANT */
         CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
         return; 
    };

    switch(subdevice) {

    case 0x00:
        if (value) {
            P0 |= (1 << line);
        } else {
            P0 &= ~(1 << line);
        }
        break;
    case 0x01:
        if (value) {
            P1 |= (1 << line);
        } else {
            P1 &= ~(1 << line);
        }
        break;
    case 0x02:
        if (value) {
            P2 |= (1 << line);
        } else {
            P2 &= ~(1 << line);
        }
        break;
    case 0x03:
        if (value) {
            P3 |= (1 << line);
        } else {
            P3 &= ~(1 << line);
        }
        break;
    default:
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return;
    }

    /* all worked OK */
    CD_LinkMsgOK();
}



/****************************************************************************/
/* DigitalOutputPort                                                        */
void CD_DOP(unsigned char *msg) large {
    
    unsigned char subdevice;
    //unsigned char line;
    unsigned char value;

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    if (CD_HexaStrTo_u8(msg+8, 2, &value)) {
         CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
         return; 
    };

    switch(subdevice) {

    case 0x00:
        P0= value;
        break;
    case 0x01:
        P1= value;
        break;
    case 0x02:
        P2= value;
        break;
    case 0x03:
        P3= value;
        break;
    default:
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return;
    }

    /* all worked OK */
    CD_LinkMsgOK();
}


/****************************************************************************/
/* DigitalOutputPort with mask                                                       */
void CD_DOM(unsigned char *msg) large {
    
    unsigned char subdevice;
    //unsigned char line;
    unsigned char value;
    unsigned char mask;
    unsigned char data val_and, data val_or;

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    /* value for the port */
    if (CD_HexaStrTo_u8(msg+8, 2, &value)) {
         CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
         return; 
    };

    /* mask for the port */
    if (CD_HexaStrTo_u8(msg+16, 2, &mask)) {
         CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
         return; 
    };

    /* precalculate values for the "and" and "or" operation to minimize glitches */
    val_and = ~mask | value;
    val_or = mask & value;

    switch(subdevice) {

    case 0x00:
        P0 &= val_and;
        P0 |= val_or;
        break;
    case 0x01:
        P1 &= val_and;
        P1 |= val_or;
        break;
    case 0x02:
        P2 &= val_and;
        P2 |= val_or;
        break;
    case 0x03:
        P3 &= val_and;
        P3 |= val_or;
        break;
    default:
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return;
    }

    /* all worked OK */
    CD_LinkMsgOK();
}

/****************************************************************************/
/* DigitalInputLine                                                         */
void CD_DIL(unsigned char *msg) large {
    
    unsigned char subdevice;
    unsigned char line;
    unsigned char value;

    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    if (CD_HexaStrTo_u8(msg+2, 2, &line)) {
        CD_LinkMsgError(CD_Err_BadValue);  /* bad line */
        return; 
    };

    switch(subdevice) {

    case 0x00:
        value = P0;
        break;
    case 0x01:
        value = P1;
        break;
    case 0x02:
        value = P2;
        break;
    case 0x03:
        value = P3;
        break;
    default:
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return;
    }


    if (value & (1<<line)) {
        CD_LinkTXStr("#!1\x0D");
    } else {
        CD_LinkTXStr("#!0\x0D");
    }
}


/****************************************************************************/
/* DigitalInputPort                                                         */
void CD_DIP(unsigned char *msg) large {
    
    unsigned char subdevice;
    unsigned char value;

    unsigned char resp[3];


    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    switch(subdevice) {

    case 0x00:
        value = P0;
        break;
    case 0x01:
        value = P1;
        break;
    case 0x02:
        value = P2;
        break;
    case 0x03:
        value = P3;
        break;
    default:
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return;
    }

    /* send response */
    CD_u8_ToHexaStr(value, 2, resp);
    CD_LinkTXStr("#!000000");
    CD_LinkTXStr(resp);
    CD_LinkTXChar('\x0D');
}
