/*
    File: CD_Link.c
    Data from/to HOST management (sending,...)
    Angel Perles. May 2009.
*/

#include "USB_Type.h"
#include "USB_CDC_UART.h"
#include <stdio.h>
#include <string.h>

#include <C8051F340.h>

//#include "CD_Link.h"
//#include "CD_Parse.h"

#include "CD_Main.h"

sbit Led1 = P2^2;   // LED='1' ON

#define RX_BUFF_SIZE    250     /* size of the receiving buffer             */
#define RX_START_MSG_NOT_RECEIVED   RX_BUFF_SIZE

static unsigned char xdata rx_buffer[RX_BUFF_SIZE+1]; /* receiver buffer    */
static unsigned char rx_cnt;                    /* bytes received           */
static unsigned char rx_msg_start;              /* message start pointer    */
static unsigned char rx_msg_end;                /* message end pointer      */


/****************************************************************************/
/* Module initilization                                                     */
void CD_LinkInit(void) {

    /* clear receive buffer */
    rx_buffer[0] = '\0';
    rx_cnt = 0;

    /* no start mesage received mark*/
    rx_msg_start = RX_START_MSG_NOT_RECEIVED;     
}

/****************************************************************************/
/* data received arrives to this function and messages are searched         */
void CD_LinkRXChar(unsigned char chr) {

    /* check for space */
    if (rx_cnt >= RX_BUFF_SIZE) {   /* no space */
        rx_cnt = 0; /* clear all data */
        rx_msg_start = RX_START_MSG_NOT_RECEIVED;
    }

    if (chr == MSG_START_CHAR) {   /* start of the message found!!         */
        rx_msg_start = rx_cnt;
    } else if (chr == MSG_END_CHAR) {  /* end of message found !!          */

        rx_msg_end = rx_cnt;
        rx_buffer[rx_cnt] = '\0';   /* force end of string                  */

        /* handle message if complete */
        if (rx_msg_start != RX_START_MSG_NOT_RECEIVED) {
            CD_ParseMsg(&rx_buffer[rx_msg_start], rx_msg_end - rx_msg_start);
        } 

        /* clear message buffer */
        rx_cnt = 0;
        rx_msg_start = RX_START_MSG_NOT_RECEIVED;

    } else {    /* save char */
        rx_buffer[rx_cnt++] = chr;
    } 
}


/****************************************************************************/
/* data byte to be transfered                                               */
void CD_LinkTXChar(unsigned char chr) {

    while(!RXReady){};  /* wait until room in the buffer                    */
    COMPutByte(chr);
}

/****************************************************************************/
/* str to be transfered                                                     */
void CD_LinkTXStr(unsigned char *str) {

    while(*str != '\0') {
        CD_LinkTXChar(*str);
        str++;
    }
}

/****************************************************************************/
/* send an error msg                                                        */
void CD_LinkMsgError(unsigned char cod) {
    
    unsigned char msg[3];

    CD_u8_ToHexaStr(cod, 2, msg);

    CD_LinkTXStr("#?");
    CD_LinkTXStr(msg);
    CD_LinkTXChar('\x0D');
}

/****************************************************************************/
/* send OK  msg                                                             */
void CD_LinkMsgOK(void) {
    CD_LinkTXStr("#!\x0D");
}
