/* CD_PWM.c 
   PulseWiddthModulated Signals
   Angel Perles. June 2009.

   used resources
    P0.0

   dependencies

   IMPORTANT:
    P2.2 also used for DigitalOutput example
    8-bits PWM, could be enhanced with 16-bit PWM mode

*/

#include <C8051F340.h>
#include "CD_Main.h"

sbit  Pin_PWM0      = P0^0;                 /* PWM output pin */ 

/****************************************************************************/
void CD_PWM_Init(void) {

   // Configure PCA time base; overflow interrupt disabled
   PCA0CN = 0x00;                      // Stop counter; clear all flags
   //PCA0MD = 0x08;                      // Use SYSCLK as time base
   PCA0MD = 0x00;       /* SYSCLK/12 as time base (48 MHz/12) */

   PCA0CPM0 = 0x42;                    // Module 0 = 8-bit PWM mode
   

   // Configure initial PWM duty cycle = 50%
   /* PCA0CPH0 = 256 - (256 * 0.5); */

   /* A 0% duty cycle may be generated by clearing the ECOMn bit to 0. */
   XBR1   |= 0x01;                  // Put CEX0 on Crossbar 
   CR      = 1;                     // Start PCA Counter

}


/****************************************************************************/
void CD_PWM(unsigned char *msg) large  {
    unsigned char subdevice;
   //unsigned int ADC_result;

    unsigned int val;


    /* parse string values */
    if (CD_HexaStrTo_u8(msg, 2, &subdevice)) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    /* test subdevice numbre */
    if (subdevice != 0x30) {
        CD_LinkMsgError(CD_Err_BadSubdevice);  /* bad subdevice */
        return; 
    };

    /* duty-cycle, from 100.0 to 0.0 without comma (tenths of percent) */
    if (CD_DecStrTo_u16(msg+2, 4, &val)) {
        CD_LinkMsgError(CD_Err_BadValue);  /* bad value */
        return; 
    };

    val /= 10;  /* should be enhanced on next versions */

    if (val == 0) { /* 0% */
        PCA0CPM0 &= ~0x40;  /* ECOM0=0 to stop generation */    
    } else {
        PCA0CPH0 = 256 - (256 * val)/100;
        PCA0CPM0 |= 0x40;  /* ECOM0=1 to enable output */    
    }
 
    CD_LinkMsgOK();
}
