/* CD_Parse.c 
  message parser
*/

#include "CD_Main.h"

#include <stdio.h>
#include <string.h>

#include <C8051F340.h>

// structura para mantener la lista de rdenes --------------------------------
struct cmdlst {
	char cmd[4];                        /* command name */
	unsigned char len;                  /* len of the message */
	void (*fun)(unsigned char *str) large;    /* function to call */
    char kk;    /* provisional */
};

struct cmdlst code cd_commands[] = {
	{"DOL",8,CD_DOL,1},                 /* Digital Output Line  */
    {"DIL",7,CD_DIL,2},                 /* Digital Input Line   */
    {"DOF",13,CD_DOF,3},
    {"DIF",5,CD_DIF,4},
    {"AIS",7,CD_AIS,5},
    {"PWM",9,CD_PWM,6},

    {"DOP",13,CD_DOP,7},          /* digital port out */
    {"DOM",21,CD_DOM,8},          /* digital port out with mask*/
    {"DIP",5,CD_DIP,9}, 

    { 0, 0, 0, 0}
};



/****************************************************************************/
/* parse received messages                                                  */
void CD_ParseMsg(unsigned char *msg, unsigned char len) {

    struct cmdlst *p_cmdlst;

    // length of the messges must be al least 3 bytes
    if (len<3) {
        CD_LinkMsgError(0x00);  // invalid order
        return;
    }

    // find command
    p_cmdlst = cd_commands;
    while (p_cmdlst->cmd[0] != 0) {
        if (strncmp(msg,p_cmdlst->cmd,3)) { /* not this */
            p_cmdlst++;
        } else {
            if (len != p_cmdlst->len) { /* bad length of the string */
                CD_LinkMsgError(0x00);  /* invalid command */
                return;    
            } else {
                break;
            }
        }
    }

    /* test if the command has ben found */
    if (p_cmdlst->cmd[0] == 0) {
       CD_LinkMsgError(0x00);  /* order not found */
       return;
    } 

    /* run the command */
    //(p_cmdlst->fun)();

    switch (p_cmdlst->kk) {
        case 1:
            CD_DOL(msg+3);
            break;    
        case 2:
            CD_DIL(msg+3);
            break; 
        case 3:
            CD_DOF(msg+3);
            break;    
        case 4:
            CD_DIF(msg+3);
            break; 
        case 5:
            CD_AIS(msg+3);
            break; 
        case 6:
            CD_PWM(msg+3);
            break;
        case 7:
            CD_DOP(msg+3);
            break;
        case 8:
            CD_DOM(msg+3);
            break;                            
        case 9:
            CD_DIP(msg+3);
            break;  
    }
}

