/*
    File: cdm_Link.c
    Data from/to HOST management (sending,...)
    Angel Perles. May 2009. May 2010.
    Version for qt (depends on qextserialport project)

*/

#include <QtCore/QCoreApplication>
#include <stdio.h>
#include <qextserialport.h>

#include <cdm_Link.h>


#include <string.h>







//#include "CD_Parse.h"

//#include "CD_Main.h"



static bool serial_ready = false;
static QextSerialPort *port = 0; // pointer to the serial port handler


/****************************************************************************/
/* Module initilization                                                     */
/* in this implementation, "params" will be de serial port identifier                */
void cdm_LinkInit(const char *params) {

    // test if serial port handler has been created and create it if necessary
    if (port == 0) {
        port = new QextSerialPort();
        port->setQueryMode(QextSerialPort::Polling);  // modo mas sencillo
        serial_ready = false;
    }

    // if port is open, then close it
    if (serial_ready) {
        port->close();
        serial_ready = false;
    }

    // configure port parameters
    port->setPortName(params);
    port->setBaudRate (BAUD9600);
    port->setDataBits(DATA_8);
    port->setStopBits(STOP_1);
    port->setParity(PAR_NONE);
    port->setFlowControl(FLOW_OFF);

    if (!port->open(QextSerialPort::ReadWrite)) {
        return (CDM_Error_UnableToOpenLink);
    }

    serial_ready = true;
    return(CDM_Error_NoError);
}


/****************************************************************************/
/* implementation of the low level link data reception         */
cdm_Error cdm_LinkReceive(char *receive_buffer, int max_bytes, int *received_bytes) {

    if (!serial_ready) {
        return(CDM_Error_LinkNotReady);
    }
    /* check for space */
    if (rx_cnt >= RX_BUFF_SIZE) {   /* no space */
        rx_cnt = 0; /* clear all data */
        rx_msg_start = RX_START_MSG_NOT_RECEIVED;
    }

    if (chr == MSG_START_CHAR) {   /* start of the message found!!         */
        rx_msg_start = rx_cnt;
    } else if (chr == MSG_END_CHAR) {  /* end of message found !!          */

        rx_msg_end = rx_cnt;
        rx_buffer[rx_cnt] = '\0';   /* force end of string                  */

        /* handle message if complete */
        if (rx_msg_start != RX_START_MSG_NOT_RECEIVED) {
            CD_ParseMsg(&rx_buffer[rx_msg_start], rx_msg_end - rx_msg_start);
        } 

        /* clear message buffer */
        rx_cnt = 0;
        rx_msg_start = RX_START_MSG_NOT_RECEIVED;

    } else {    /* save char */
        rx_buffer[rx_cnt++] = chr;
    } 
}


/****************************************************************************/
/* data byte to be transfered                                               */
void CD_LinkTXChar(unsigned char chr) {

    while(!RXReady){};  /* wait until room in the buffer                    */
    COMPutByte(chr);
}

/****************************************************************************/
/* str to be transfered                                                     */
void CD_LinkTXStr(unsigned char *str) {

    while(*str != '\0') {
        CD_LinkTXChar(*str);
        str++;
    }
}

/****************************************************************************/
/* send an error msg                                                        */
void CD_LinkMsgError(unsigned char cod) {
    
    unsigned char msg[3];

    CD_u8_ToHexaStr(cod, 2, msg);

    CD_LinkTXStr("#?");
    CD_LinkTXStr(msg);
    CD_LinkTXChar('\x0D');
}

/****************************************************************************/
/* send OK  msg                                                             */
void CD_LinkMsgOK(void) {
    CD_LinkTXStr("#!\x0D");
}
