#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QLabel>
#include <QTimer>

#include <cdm_Link.h>
#include <cdm_DAQ.h>


#include <QtSvgDialGauge>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // Tachometer
    gauge = new QtSvgDialGauge(this);
    ui->vlKK->addWidget(gauge);
    gauge->setSkin("Tachometer");
    gauge->setNeedleOrigin(0.486, 0.466);
    gauge->setMinimum(0);
    gauge->setMaximum(120);
    gauge->setStartAngle(-130);
    gauge->setEndAngle(133);
    gauge->setValue(0);
    gauge->setMaximumSize(200, 200);
    //gauge->setValue(100);

    lbDialValue = new QLabel("aci el que valga el dial");
    //lbDialValue->setAlignment()
    ui->vlKK->addWidget(lbDialValue);

    //ui->statusBar->

    timer = new QTimer();

    connect(ui->btInit,SIGNAL(clicked()),this,SLOT(handle_btInit()));
    connect(ui->btSend,SIGNAL(clicked()),this,SLOT(handle_btSend()));
    connect(ui->btReceive,SIGNAL(clicked()),this,SLOT(handle_btReceive()));
    connect(ui->btTest,SIGNAL(clicked()),this,SLOT(handle_btTest()));
    connect(ui->btDOL0,SIGNAL(clicked()),this,SLOT(handle_btDOL0()));
    connect(ui->btDOL1,SIGNAL(clicked()),this,SLOT(handle_btDOL1()));
    connect(ui->vsTestDial,SIGNAL(valueChanged (int)),this,SLOT(handle_vsTestDial(int)));
    connect(timer,SIGNAL(timeout()),this,SLOT(handle_timer()));

    connect(ui->sbPWM,SIGNAL(valueChanged(int)),this,SLOT(handle_sbPWM(int)));

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::handle_btInit(void) {

    cdm_LinkInit(ui->lePortName->text().toAscii().data());

}

void MainWindow::handle_btSend(void) {

    cdm_LinkTXStr(ui->leSend->text().toAscii().data());

}

void MainWindow::handle_btReceive(void) {

    char buff[1000];
    int received_bytes;

    cdm_LinkRXStr(buff,3,&received_bytes);
    if (received_bytes > 0) {
        buff[received_bytes] = '\0';
        ui->leReceive->setText(buff);
    } else {
        ui->leReceive->setText("---");
    }
}

void MainWindow::handle_btTest(void) {

    //char msg[100];

    timer->setInterval(1000);
    timer->start();

    /*
    cdm_Error cdm_result;
    double value;

    cdm_result = cdm_DAQReadDigitalFrecuency(10, &value);
    if (cdm_result == CDM_Error_NoError) {
        //ui->btTest->setText(msg);
        setDial(value);
    } else if (cdm_result == CDM_Error_Timeout) {
        ui->btTest->setText("<timeout>");
    } else {
        ui->btTest->setText("Something peta!");
    }
    */


}

void MainWindow::handle_btDOL0(void) {

    cdm_Error cdm_result;

    cdm_result = cdm_DAQWriteDigitalLine(2,2,0);  // subdevice=2, line=2, value=0

    if (cdm_result == CDM_Error_NoError) {
        ui->btTest->setText("Done!");
    } else if (cdm_result == CDM_Error_Timeout) {
        ui->btTest->setText("<timeout>");
    } else {
        ui->btTest->setText("Something peta!");
    }
}


void MainWindow::handle_btDOL1(void) {

    cdm_Error cdm_result;

    cdm_result = cdm_DAQWriteDigitalLine(2,2,1);  // subdevice=2, line=2, value=1

    if (cdm_result == CDM_Error_NoError) {
        ui->btTest->setText("Done!");
    } else if (cdm_result == CDM_Error_Timeout) {
        ui->btTest->setText("<timeout>");
    } else {
        ui->btTest->setText("Something peta!");
    }
}


void MainWindow::handle_vsTestDial(int value) {

    setDial(value);

}

void MainWindow::setDial(double value) {

    QString a;

    a.sprintf("%6.2lf Hz", value);
    lbDialValue->setText(a);
    gauge->setValue(value);

}

void MainWindow::handle_sbPWM(int value) {
    QString a;
    double dutty;
    cdm_Error cdm_result;

    dutty = value /10.0;
    a.sprintf("%5.1lf %%",dutty);
    ui->lineEdit->setText(a);
    cdm_result = cdm_DAQWritePWM(30, dutty);
    //ui->lineEdit


}

void MainWindow::handle_timer(void) {

    cdm_Error cdm_result;
    double value;

    cdm_result = cdm_DAQReadDigitalFrecuency(10, &value);
    if (cdm_result == CDM_Error_NoError) {
        //ui->btTest->setText(msg);
        setDial(value);
    } else if (cdm_result == CDM_Error_Timeout) {
        ui->btTest->setText("<timeout>");
    } else {
        ui->btTest->setText("Something peta!");
    }

}
